@@ -6,7 +6,7 @@ use Moo;
 use MooX::Types::MooseLike::Base qw(ArrayRef Str);
 use namespace::autoclean;
 
-our $VERSION = '2.002';
+our $VERSION = '2.004';
 
 extends qw(
     Locale::TextDomain::OO::Extract::Base::RegexBasedExtractor
@@ -23,8 +23,9 @@ my $category_rule
     = my $text_rule
     = [
         [
+            # "text with 0 .. n escaped chars"
             qr{
-                ["]
+                \s* ["]
                 (
                     [^\\"]*              # normal text
                     (?: \\ . [^\\"]* )*  # maybe followed by escaped char and normal text
@@ -34,8 +35,9 @@ my $category_rule
         ],
         'or',
         [
+            # 'text with 0 .. n escaped chars'
             qr{
-                [']
+                \s* [']
                 (
                     [^\\']*              # normal text
                     (?: \\ . [^\\']* )*  # maybe followed by escaped char and normal text
@@ -44,8 +46,8 @@ my $category_rule
             }xms,
         ],
     ];
-my $comma_rule = qr{ \s* , \s* }xms;
-my $count_rule = qr{ \s* ( [^,)]+ ) \s* }xms;
+my $comma_rule = qr{ \s* [,] }xms;
+my $count_rule = qr{ \s* ( [^,)]+ ) }xms;
 my $close_rule = qr{ \s* [,]? \s* ( [^)]* ) [)] }xms;
 
 my $start_rule = qr{
@@ -62,368 +64,592 @@ my $start_rule = qr{
 my $rules = [
     # loc_, _, __
     [
-        qr{ \b N? (?: loc_ | __? ) ( x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( x? ) \s* [(] }xms,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( n x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( n x? ) \s* [(] }xms,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( p x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( p x? ) \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( np x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( np x? ) \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
     ],
 
     # loc_d, _d, __d
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( d x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( d x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( dn x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( dn x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( dp x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( dp x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( dnp x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( dnp x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
     ],
 
     # loc_c, _c, __c
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( c x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( c x? ) \s* [(] }xms,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( cn x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( cn x? ) \s* [(] }xms,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( cp x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( cp x? ) \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( cnp x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( cnp x? ) \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
 
     # loc_dc, _dc, __dc
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( dc x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( dc x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( dcn x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( dcn x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( dcp x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( dcp x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __? ) ( dcnp x? ) \s* [(] \s* }xms,
+        qr{ \b N? (?: loc_ | __? ) ( dcnp x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
 
     # gettext
     'or',
     [
-        qr{ \b N? () gettext \s* [(] \s* }xms,
+        qr{ \b N? () gettext \s* [(] }xms,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( n ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( n ) gettext \s* [(] }xms,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( p ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( p ) gettext \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( np ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( np ) gettext \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
     ],
 
     # dgettext
     'or',
     [
-        qr{ \b N? ( d ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( d ) gettext \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( dn ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( dn ) gettext \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( dp ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( dp ) gettext \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( dnp ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( dnp ) gettext \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
     ],
 
     # cgettext
     'or',
     [
-        qr{ \b N? ( c ) gettext [(] \s* }xms,
+        qr{ \b N? ( c ) gettext \s* [(] }xms,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( cn ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( cn ) gettext \s* [(] }xms,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( cp ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( cp ) gettext \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( cnp ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( cnp ) gettext \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
 
     # dcgettext
     'or',
     [
-        qr{ \b N? ( dc ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( dc ) gettext \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( dcn ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( dcn ) gettext \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( dcp ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( dcp ) gettext \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
     'or',
     [
-        qr{ \b N? ( dcnp ) gettext \s* [(] \s* }xms,
+        qr{ \b N? ( dcnp ) gettext \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
     ],
 ];
@@ -444,21 +670,20 @@ sub preprocess {
     return $self;
 }
 
-my %char_of = (
-    b => "\b",
-    f => "\f",
-    n => "\n",
-    r => "\r",
-    t => "\t",
-);
-
-sub _interpolate_escape_sequence {
-    my ($self, $string) = @_;
+my $interpolate_escape_sequence = sub {
+    my $string = shift;
 
     # nothing to interpolate
     defined $string
         or return;
 
+    my %char_of = (
+        b    => "\b",
+        f    => "\f",
+        n    => "\n",
+        r    => "\r",
+        t    => "\t",
+    );
     ## no critic (ComplexRegexes)
     $string =~ s{
         \\
@@ -477,16 +702,16 @@ sub _interpolate_escape_sequence {
                   # anything else that needs no escape
         )
     }{
-       $1   ? $char_of{$1}
-       : $2 ? chr hex $2
-       : $3 ? chr hex $3
-       : $4 ? chr oct $4
-       :      $5
+        $1   ? $char_of{$1}
+        : $2 ? chr hex $2
+        : $3 ? chr hex $3
+        : $4 ? chr oct $4
+        :      $5
     }xmsge;
     ## use critic (ComplexRegexes)
 
     return $string;
-}
+};
 
 sub stack_item_mapping {
     my $self = shift;
@@ -500,30 +725,24 @@ sub stack_item_mapping {
     $self->add_message({
         reference    => ( sprintf '%s:%s', $self->filename, $_->{line_number} ),
         domain       => $extra_parameter =~ m{ d }xms
-            ? scalar $self->_interpolate_escape_sequence( shift @{$match} )
+            ? scalar $interpolate_escape_sequence->( shift @{$match} )
             : $self->domain,
         msgctxt      => $extra_parameter =~ m{ p }xms
-            ? scalar $self->_interpolate_escape_sequence( shift @{$match} )
+            ? scalar $interpolate_escape_sequence->( shift @{$match} )
             : undef,
-        msgid        => scalar $self->_interpolate_escape_sequence(
-            shift @{$match},
-        ),
+        msgid        => scalar $interpolate_escape_sequence->( shift @{$match} ),
         msgid_plural => $extra_parameter =~ m{ n }xms
             ? do {
-                my $plural = $self->_interpolate_escape_sequence(
-                    shift @{$match},
-                );
-                $count = $self->_interpolate_escape_sequence( shift @{$match} );
+                my $plural = $interpolate_escape_sequence->( shift @{$match} );
+                $count = shift @{$match};
                 $plural;
             }
             : undef,
         category     => $extra_parameter =~ m{ c }xms
-            ? scalar $self->_interpolate_escape_sequence( shift @{$match} )
+            ? scalar $interpolate_escape_sequence->( shift @{$match} )
             : $self->category,
         automatic    => do {
-            my $placeholders = $self->_interpolate_escape_sequence(
-                shift @{$match},
-            );
+            my $placeholders = shift @{$match};
             my $string = join ', ', map { ## no critic (MutatingListFunctions)
                 defined $_
                 ? do {
@@ -571,7 +790,7 @@ $HeadURL: svn+ssh://steffenw@svn.code.sf.net/p/perl-gettext-oo/code/extract/trun
 
 =head1 VERSION
 
-2.002
+2.004
 
 =head1 DESCRIPTION
 
@@ -791,7 +1010,7 @@ Steffen Winkler
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2009 - 2014,
+Copyright (c) 2009 - 2015,
 Steffen Winkler
 C<< <steffenw at cpan.org> >>.
 All rights reserved.
