@@ -7,7 +7,7 @@ use Moo;
 use MooX::Types::MooseLike::Base qw(ArrayRef Str);
 use namespace::autoclean;
 
-our $VERSION = '2.002';
+our $VERSION = '2.004';
 
 extends qw(
     Locale::TextDomain::OO::Extract::Base::RegexBasedExtractor
@@ -27,7 +27,7 @@ my $category_rule
         [
             # 'text with 0 .. n escaped chars'
             qr{
-                [']
+                \s* ( ['] )
                 (
                     [^\\']*              # normal text
                     (?: \\ . [^\\']* )*  # maybe followed by escaped char and normal text
@@ -36,16 +36,28 @@ my $category_rule
             }xms,
         ],
         'or',
+        [
+            # "text with 0 .. n escaped chars"
+            qr{
+                \s* ( ["] )
+                (
+                    [^\\"]*              # normal text
+                    (?: \\ . [^\\"]* )*  # maybe followed by escaped char and normal text
+                )
+                ["]
+            }xms,
+        ],
+        'or',
         [
             # q{text with 0 .. n {placeholders} and/or 0 .. n escaped chars}
             ## no critic (EscapedMetacharacters)
             qr{
-                [q] \{                # q curly bracked quoted
+                \s* ( qq? ) \{        # q curly bracket quoted
                 (
                     (?:
                         [^\{\}\\]     # normal text
                         | \\ .        # escaped char
-                        | \{ (?-1) \} # any paars of curly brackets with the same stuff inside
+                        | \{ (?-1) \} # any pairs of curly brackets with the same stuff inside
                     )*
                 )
                 \}                    # end of quote
@@ -53,8 +65,8 @@ my $category_rule
             ## use critic (EscapedMetacharacters)
         ],
     ];
-my $comma_rule = qr{ \s* , \s* }xms;
-my $count_rule = qr{ \s* ( [^,)]+ ) \s* }xms;
+my $comma_rule = qr{ \s* [,] }xms;
+my $count_rule = qr{ \s* ( [^,)]+ ) }xms;
 my $close_rule = qr{ \s* [,]? \s* ( [^)]* ) [)] }xms;
 
 ## no critic (Complex Regexes)
@@ -82,241 +94,417 @@ my $start_rule = qr{
 my $rules = [
     # loc_, __
     [
-        qr{ \b N? (?: loc_ | __ ) ( x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( x? ) \s* [(] }xms,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( n x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( n x? ) \s* [(] }xms,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( p x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( p x? ) \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( np x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( np x? ) \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
+        'end',
     ],
 
     # loc_d, __d
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( d x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( d x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( dn x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( dn x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( dp x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( dp x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( dnp x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( dnp x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $close_rule,
+        'end',
     ],
 
     # loc_c, __c
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( c x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( c x? ) \s* [(] }xms,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( cn x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( cn x? ) \s* [(] }xms,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( cp x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( cp x? ) \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( cnp x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( cnp x? ) \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
+        'end',
     ],
 
     # loc_dc, __dc
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( dc x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( dc x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( dcn x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( dcn x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( dcp x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( dcp x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? (?: loc_ | __ ) ( dcnp x? ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? (?: loc_ | __ ) ( dcnp x? ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $singular_rule,
+        'and',
         $comma_rule,
+        'and',
         $plural_rule,
+        'and',
         $comma_rule,
+        'and',
         $count_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'and',
         $close_rule,
+        'end',
     ],
 
     # maketext loc, localize, localize
     'or',
     [
-        qr{ \b N? loc (?: ali[sz]e )? (?: _m )? () \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? loc (?: ali[sz]e )? (?: _m )? () \s* [(] }xms,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? loc (?: ali[sz]e )? _m ( p ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? loc (?: ali[sz]e )? _m ( p ) \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
+        'end',
     ],
 
     # maketext
     'or',
     [
-        qr{ \b N? maketext () \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? maketext () \s* [(] }xms,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b N? maketext_ ( p ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b N? maketext_ ( p ) \s* [(] }xms,
+        'and',
         $context_rule,
+        'and',
         $comma_rule,
+        'and',
         $text_rule,
+        'and',
         $close_rule,
+        'end',
     ],
 
     # begin
     'or',
     [
-        qr{ \b (?: loc_ | __ ) ( begin ) _ ( [dc] ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b (?: loc_ | __ ) ( begin ) _ ( [dc] ) \s* [(] }xms,
+        'and',
         $domain_or_category_rule,
+        'end',
     ],
     'or',
     [
-        qr{ \b (?: loc_ | __ ) ( begin ) [_] ( dc ) \s* [(] \s* }xms,
+        'begin',
+        qr{ \b (?: loc_ | __ ) ( begin ) [_] ( dc ) \s* [(] }xms,
+        'and',
         $domain_rule,
+        'and',
         $comma_rule,
+        'and',
         $category_rule,
+        'end',
     ],
 
     # end
     'or',
     [
+        'begin',
         qr{ \b (?: loc_ | __ ) ( end ) [_] ( [dc] ) \b }xms,
+        'end',
     ],
     'or',
     [
+        'begin',
         qr{ \b (?: loc_ | __ ) ( end ) [_] ( dc ) \b }xms,
+        'end',
     ],
 ];
 
@@ -328,25 +516,105 @@ sub preprocess {
 
     my ($is_pod, $is_end);
     ${$content_ref} = join "\n", map {
-        $_ eq '__END__'      ? do { $is_end = 1; q{} }
-        : $is_end            ? ()
-        : m{ = ( \w+ ) }xms  ? (
+        $_ eq '__END__'        ? do { $is_end = 1; q{} }
+        : $is_end              ? ()
+        : m{ $ = ( \w+ ) }xms  ? (
             lc $1 eq 'cut'
             ? do { $is_pod = 0; q{} }
             : do { $is_pod = 1; q{} }
         )
-        : $is_pod            ? q{}
+        : $is_pod              ? q{}
         : $_;
     } split m{ \r? \n }xms, ${$content_ref};
 
+    # replace heredoc's without killing the line number
+    REPLACE: {
+        ${$content_ref} =~ s{
+            << \s* ' ( \w+ ) ' ( [^\n]* ) \n
+            ( .*? )
+            ^ \1 $
+        }
+        {
+            qq{\n'}
+            . do { my $text = $3; $text =~ s{'}{\\'}xmsg; $text }
+            . q{'}
+            . $2
+        }xmsge and redo REPLACE;
+    }
+    REPLACE: {
+        ${$content_ref} =~ s{
+            << \s* ( ["]? ) ( \w+ ) \1 ( [^\n]* ) \n
+            ( .*? )
+            ^ \2 $
+        }
+        {
+            qq{\n"}
+            . do { my $text = $4; $text =~ s{"}{\\"}xmsg; $text }
+            . q{"}
+            . $3
+        }xmsge and redo REPLACE;
+    }
+
     return;
 }
 
+my $interpolate_escape_sequence = sub {
+    my ( $quot, $string ) = @_;
+
+    # nothing to interpolate
+    defined $string
+        or return;
+    defined $quot
+        or return;
+    my $is_interpolate = $quot eq q{"} || $quot eq 'qq';
+    if ( ! $is_interpolate ) {
+        if ( $quot eq q{'} ) {
+            $string =~ s{ \\ ( ['] ) }{$1}xmsg;
+            return $string;
+        }
+        if ( $quot eq q{q} ) {
+            $string =~ s{ \\ ( [\{\}] ) }{$1}xmsg;
+            return $string;
+        }
+    }
+
+    my %char_of = (
+        b => "\b",
+        f => "\f",
+        n => "\n",
+        r => "\r",
+        t => "\t",
+    );
+    ## no critic (ComplexRegexes)
+    $string =~ s{
+        \\
+        (?:
+            ( [bfnrt] ) # Backspace
+                        # Form feed
+                        # New line
+                        # Carriage return
+                        # Horizontal tab
+            | ( [xN] )  # do not handle \x.., \x{...}, \N{...}
+            | (.)       # Backslash itself
+                        # Single quotation mark
+                        # Double quotation mark
+                        # anything else that needs no escape
+        )
+    }{
+        $1   ? $char_of{$1}
+        : $2 ? "\\$2"
+        :      $3
+    }xmsge;
+    ## use critic (ComplexRegexes)
+
+    return $string;
+};
+
 sub stack_item_mapping {
     my $self = shift;
 
     my $match = $_->{match};
-    # The chars after __ were stored to make a decision now.
+    # The chars e.g. after loc_ were stored to make a decision now.
     my $extra_parameter = shift @{$match};
     @{$match}
         or return;
@@ -355,17 +623,25 @@ sub stack_item_mapping {
         {
             d => sub {
                 push @{ $self->domain_stack }, $self->domain;
-                $self->domain( shift @{$match} );
+                $self->domain(
+                    scalar $interpolate_escape_sequence->( splice @{$match}, 0, 2 ),
+                );
             },
             c => sub {
                 push @{ $self->category_stack }, $self->category;
-                $self->category( shift @{$match} );
+                $self->category(
+                    scalar $interpolate_escape_sequence->( splice @{$match}, 0, 2 ),
+                );
             },
             dc => sub {
                 push @{ $self->domain_stack }, $self->domain;
-                $self->domain( shift @{$match} );
+                $self->domain(
+                    scalar $interpolate_escape_sequence->( splice @{$match}, 0, 2 ),
+                );
                 push @{ $self->category_stack }, $self->category;
-                $self->category( shift @{$match} );
+                $self->category(
+                    scalar $interpolate_escape_sequence->( splice @{$match}, 0, 2 ),
+                );
             },
         }->{ shift @{$match} }->();
         return;
@@ -398,21 +674,21 @@ sub stack_item_mapping {
     $self->add_message({
         reference    => ( sprintf '%s:%s', $self->filename, $_->{line_number} ),
         domain       => $extra_parameter =~ m{ d }xms
-            ? shift @{$match}
+            ? scalar $interpolate_escape_sequence->( splice @{$match}, 0, 2 )
             : $self->domain,
         msgctxt      => $extra_parameter =~ m{ p }xms
-            ? shift @{$match}
+            ? scalar $interpolate_escape_sequence->( splice @{$match}, 0, 2 )
             : undef,
-        msgid        => shift @{$match},
+        msgid        => scalar $interpolate_escape_sequence->( splice @{$match}, 0, 2 ),
         msgid_plural => $extra_parameter =~ m{ n }xms
             ? do {
-                my $plural = shift @{$match};
+                my $plural = $interpolate_escape_sequence->( splice @{$match}, 0, 2 );
                 $count = shift @{$match};
                 $plural;
             }
             : undef,
         category     => $extra_parameter =~ m{ c }xms
-            ? shift @{$match}
+            ? scalar $interpolate_escape_sequence->( splice @{$match}, 0, 2 )
             : $self->category,
         automatic    => do {
             my $placeholders = shift @{$match};
@@ -463,7 +739,7 @@ $HeadURL: svn+ssh://steffenw@svn.code.sf.net/p/perl-gettext-oo/code/extract/trun
 
 =head1 VERSION
 
-2.002
+2.004
 
 =head1 DESCRIPTION
 
@@ -655,7 +931,7 @@ Steffen Winkler
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2009 - 2014,
+Copyright (c) 2009 - 2015,
 Steffen Winkler
 C<< <steffenw at cpan.org> >>.
 All rights reserved.
