@@ -12,7 +12,7 @@ use MooX::StrictConstructor;
 use MooX::Types::MooseLike::Base qw(HashRef Str);
 use namespace::autoclean;
 
-our $VERSION = '2.003';
+our $VERSION = '2.004';
 
 has category => (
     is      => 'rw',
@@ -52,6 +52,18 @@ has lexicon_ref => (
     default => sub { {} },
 );
 
+sub clear {
+    my $self = shift;
+
+    $self->category( q{} );
+    $self->domain( q{} );
+    $self->language('i-default');
+    $self->project(undef);
+    $self->lexicon_ref( {} );
+
+    return;
+}
+
 sub slurp {
     my ( $self, $filename ) = @_;
 
@@ -183,7 +195,19 @@ sub spew {
         messages => [
             map { ## no critic (ComplexMappings)
                 my $return_ref = $key_util->join_message( $_, $message_ref->{$_} );
-                delete @{$return_ref}{ qw( nplurals plural ) };
+                delete @{$return_ref}{
+                    grep {
+                        ! m{
+                            \A (?:
+                                msgctxt
+                                | msgid
+                                | msgid_plural
+                                | msgstr
+                                | msgstr_plural
+                            ) \z
+                        }xms;
+                    } keys %{$return_ref}
+                };
                 $return_ref;
             }
             sort
@@ -205,13 +229,13 @@ __END__
 
 Locale::TextDomain::OO::Extract::Process::Plugin::MO - MO file plugin
 
-$Id: MO.pm 561 2014-11-11 16:12:48Z steffenw $
+$Id: MO.pm 565 2014-11-11 21:38:55Z steffenw $
 
 $HeadURL: svn+ssh://steffenw@svn.code.sf.net/p/perl-gettext-oo/code/extract/trunk/lib/Locale/TextDomain/OO/Extract/Process/Plugin/MO.pm $
 
 =head1 VERSION
 
-2.003
+2.004
 
 =head1 SYNOPSIS
 
@@ -250,6 +274,12 @@ Write MO file from lexicon_ref.
 
     $self->spew($filename);
 
+=head2 method clear
+
+Back to defaults to run the next file.
+
+    $self->clear;
+
 =head1 EXAMPLE
 
 Inside of this distribution is a directory named example.
@@ -301,7 +331,7 @@ Steffen Winkler
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2014,
+Copyright (c) 2014 - 2015,
 Steffen Winkler
 C<< <steffenw at cpan.org> >>.
 All rights reserved.
