@@ -12,7 +12,7 @@ use MooX::StrictConstructor;
 use MooX::Types::MooseLike::Base qw(HashRef Str);
 use namespace::autoclean;
 
-our $VERSION = '2.003';
+our $VERSION = '2.004';
 
 has category => (
     is      => 'rw',
@@ -52,6 +52,18 @@ has lexicon_ref => (
     default => sub { {} },
 );
 
+sub clear {
+    my $self = shift;
+
+    $self->category( q{} );
+    $self->domain( q{} );
+    $self->language('i-default');
+    $self->project(undef);
+    $self->lexicon_ref( {} );
+
+    return;
+}
+
 sub slurp {
     my ( $self, $filename ) = @_;
 
@@ -260,14 +272,22 @@ sub spew {
                                 '-msgstr_n' => {
                                     map { ## no critic (ComplexMappings)
                                         my $value = $po_data->{msgstr_plural}->[$_];
-                                        defined $value
-                                            ? ( $_ => scalar $encode_code->($value) )
-                                            : ();
+                                        ( 
+                                            $_ => scalar $encode_code->(
+                                                defined $value ? $value : q{},
+                                            ),
+                                        );
                                     }
                                     0 .. ( $nplurals - 1 )
                                 },
                             )
-                            : ( '-msgstr' => scalar $encode_code->( $po_data->{msgstr} ) )
+                            : (
+                                '-msgstr' => scalar $encode_code->(
+                                    defined $po_data->{msgstr}
+                                        ? $po_data->{msgstr}
+                                        : q{}
+                                ),
+                            )
                     ),
                 );
             }
@@ -288,13 +308,13 @@ __END__
 
 Locale::TextDomain::OO::Extract::Process::Plugin::PO - MO file plugin
 
-$Id: PO.pm 561 2014-11-11 16:12:48Z steffenw $
+$Id: PO.pm 565 2014-11-11 21:38:55Z steffenw $
 
 $HeadURL: svn+ssh://steffenw@svn.code.sf.net/p/perl-gettext-oo/code/extract/trunk/lib/Locale/TextDomain/OO/Extract/Process/Plugin/PO.pm $
 
 =head1 VERSION
 
-2.003
+2.004
 
 =head1 SYNOPSIS
 
@@ -333,6 +353,12 @@ Write PO file from lexicon_ref.
 
     $self->spew($filename);
 
+=head2 method clear
+
+Back to defaults to run the next file.
+
+    $self->clear;
+
 =head1 EXAMPLE
 
 Inside of this distribution is a directory named example.
@@ -384,7 +410,7 @@ Steffen Winkler
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2014,
+Copyright (c) 2014 - 2015,
 Steffen Winkler
 C<< <steffenw at cpan.org> >>.
 All rights reserved.
